"""
Internal management commands of the **shelter** library.
"""


SHELTER_MANAGEMENT_COMMANDS = (
    'shelter.commands.devserver.DevServer',
    'shelter.commands.runserver.RunServer',
    'shelter.commands.shell.Shell',
    'shelter.commands.showconfig.ShowConfig',
    'shelter.commands.startproject.StartProject',
)
"""
Library internal management commands.
"""



"""
Module :module:`shelter.core.config` provides base class which
encapsulates configuration.
"""

import collections
import logging.config
import sys

import six

from shelter.core.cmdlineparser import argument
from shelter.core.context import Context
from shelter.utils.imports import import_object
from shelter.utils.net import parse_host

__all__ = ['Config', 'argument']

BASE_LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'default': {
            'format': '%(asctime)s %(name)s %(levelname)s %(message)s',
            'datefmt': '%Y-%m-%d %H:%M:%S',
        },
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'level': 'NOTSET',
            'formatter': 'default',
        },
    },
    'root': {
        'handlers': ['console'],
        'level': 'INFO',
    },
}


class Config(object):
    """
    Class which encapsulates configuration. It joins options from
    settings module and command line arguments. *settings* is a Python's
    module defined by either **SHELTER_SETTINGS_MODULE** environment
    variable or **-s/--settings** command line argument and *args_parser*
    is a :class:`argparse.Namespace` instance.
    """

    Interface = collections.namedtuple(
        'Interface',
        ['name', 'host', 'port', 'unix_socket', 'processes', 'urls']
    )
    """
    Container which encapsulates arguments of the interface.
    """

    arguments = ()
    """
    Command line arguments of the Config class.
    """

    def __init__(self, settings, args_parser):
        self._settings = settings
        self._args_parser = args_parser
        self._cached_values = {}
        self.initialize()

    def __repr__(self):
        return "<{}.{}: {:#x}>".format(
            self.__class__.__module__, self.__class__.__name__, id(self)
        )

    def initialize(self):
        """
        Initialize instance attributes. You can override this method in
        the subclasses.
        """
        pass

    def configure_logging(self):
        """
        Configure Python's logging according to configuration *config*.
        """
        logging.config.dictConfig(self.logging or BASE_LOGGING)

    def get_config_items(self):
        """
        Return current configuration as a :class:`tuple` with
        option-value pairs.

        ::
            (('option1', value1), ('option2', value2))
        """
        return (
            ('settings', self.settings),
            ('context_class', self.context_class),
            ('interfaces', self.interfaces),
            ('logging', self.logging),
            ('name', self.name),
            ('init_handler', self.init_handler),
            ('sigusr1_handler', self.sigusr1_handler),
            ('sigusr2_handler', self.sigusr2_handler),
        )

    @property
    def settings(self):
        """
        Settings module of the application.
        """
        return self._settings

    @property
    def args_parser(self):
        """
        Command line arguments as a **argparse**.
        """
        return self._args_parser

    @property
    def context_class(self):
        """
        Context as a :class:`shelter.core.context.Context` class or subclass.
        """
        if 'context_class' not in self._cached_values:
            context_cls_name = getattr(self.settings, 'CONTEXT_CLASS', '')
            if context_cls_name:
                context_class = import_object(context_cls_name)
            else:
                context_class = Context
            self._cached_values['context_class'] = context_class
        return self._cached_values['context_class']

    @property
    def interfaces(self):
        """
        Interfaces as a :class:`list`of the
        :class:`shelter.core.config.Config.Interface` instances.
        """
        if 'interfaces' not in self._cached_values:
            self._cached_values['interfaces'] = []
            for name, interface in six.iteritems(self.settings.INTERFACES):
                listen = interface.get('LISTEN')
                unix_socket = interface.get('UNIX_SOCKET')
                if not listen and not unix_socket:
                    raise ValueError(
                        'Interface MUST listen either on TCP '
                        'or UNIX socket or both')
                host, port = parse_host(listen) if listen else (None, None)
                processes = int(interface.get('PROCESSES', 1))
                urls_obj_name = interface.get('URLS', '')
                if urls_obj_name:
                    urls = import_object(urls_obj_name)
                else:
                    urls = ()
                self._cached_values['interfaces'].append(
                    self.Interface(
                        name, host, port, unix_socket, processes, urls)
                )
        return self._cached_values['interfaces']

    @property
    def logging(self):
        """
        *Python's logging* configuration or :const:`None`.
        """
        return getattr(self.settings, 'LOGGING', None)

    @property
    def command_name(self):
        """
        Name of the current management command.
        """
        return self._args_parser.action

    @property
    def name(self):
        """
        Application name. It's used as a process name.
        """
        return getattr(self.settings, 'NAME', sys.argv[0])

    @property
    def init_handler(self):
        """
        Either name of the function or :class:`list` containing functions
        names which will be run during initialization of the applicationon.
        :const:`None` when no init handler.
        """
        return getattr(self.settings, 'INIT_HANDLER', None)

    @property
    def sigusr1_handler(self):
        """
        Either function (handler) which will be run on SIGUSR1 or
        :const:`None` when no signal handler.
        """
        return getattr(self.settings, 'SIGUSR1_HANDLER', None)

    @property
    def sigusr2_handler(self):
        """
        Either function (handler) which will be run on SIGUSR2 or
        :const:`None` when no signal handler.
        """
        return getattr(self.settings, 'SIGUSR2_HANDLER', None)

    @property
    def app_settings_handler(self):
        """
        Either function which will be called when instance of the Tornado
        application is created or :const:`None` when no handler.. Return
        :class:`dict` which is passed as *\*\*settings* argument into
        ``tornado.web.Application`` constructor.
        """
        return getattr(self.settings, 'APP_SETTINGS_HANDLER', None)



"""
Module :module:`shelter.core.processes` provides an ancestor and
functionality for writing service processes.
"""

import ctypes
import logging
import multiprocessing
import os
import signal
import time

import setproctitle

from shelter.core.constants import SERVICE_PROCESS

__all__ = ['BaseProcess']


class BaseProcess(multiprocessing.Process):
    """
    Ancestor for service processes. Adjust :attribute:`interval` attribute
    and override method :meth:`loop` which is repeatedly called every
    :attribute:`interval` seconds.
    """

    interval = 0
    """
    Interval in seconds. After this time :meth:`loop` method is
    repeatedly called.
    """

    def __init__(self, context):
        super(BaseProcess, self).__init__()

        self._parent_pid = os.getpid()
        self._ready = multiprocessing.Value(ctypes.c_bool, False)
        self._stop_event = multiprocessing.Event()

        self.context = context
        self.logger = logging.getLogger(
            "{:s}.{:s}".format(__name__, self.__class__.__name__))

        self.initialize()

    def initialize(self):
        """
        Initialize instance attributes. You can override this method in
        the subclasses.
        """
        pass

    @property
    def ready(self):
        """
        :const:`True` when service process has been started successfully,
        else :const:`False`.
        """
        return self._ready.value

    def stop(self):
        """
        Set stop flag. :meth:`run` method checks this flag and if it is
        :const:`True`, service process will be stopped.
        """
        self._stop_event.set()

    def run(self):
        """
        Child process. Repeatedly call :meth:`loop` method every
        :attribute:`interval` seconds.
        """
        setproctitle.setproctitle("{:s}: {:s}".format(
            self.context.config.name, self.__class__.__name__))
        self.logger.info(
            "Worker '%s' has been started with pid %d",
            self.__class__.__name__, os.getpid())

        # Register SIGINT handler which will exit service process
        def sigint_handler(unused_signum, unused_frame):
            """
            Exit service process when SIGINT is reached.
            """
            self.stop()
        signal.signal(signal.SIGINT, sigint_handler)

        # Initialize logging
        self.context.config.configure_logging()
        # Initialize child
        self.context.initialize_child(SERVICE_PROCESS, process=self)

        next_loop_time = 0
        while 1:
            # Exit if pid of the parent process has changed (parent process
            # has exited and init is new parent) or if stop flag is set.
            if os.getppid() != self._parent_pid or self._stop_event.is_set():
                break
            # Repeatedly call loop method. After first call set ready flag.
            if time.time() >= next_loop_time:
                try:
                    self.loop()
                except Exception:
                    self.logger.exception(
                        "Worker '%s' failed", self.__class__.__name__)
                else:
                    if not next_loop_time and not self.ready:
                        self._ready.value = True
                next_loop_time = time.time() + self.interval
            else:
                time.sleep(0.25)

    def loop(self):
        """
        Repeatedly in interval :attribute:`interval` do code in this
        method. It is an abstract method, override it in subclasses.
        """
        raise NotImplementedError



"""
Command line parser helpers.
"""

import argparse
import sys

from gettext import gettext as _


def argument(*args, **kwargs):
    """
    Return function's arguments how single command line argument
    should be parsed. *args* a *kwargs* have the same meaning as a
    :method:`argparse.ArgumentParser.add_argument` method.
    """
    return args, kwargs


class ArgumentParser(argparse.ArgumentParser):
    """
    Extends :class:`ArgumentParser` from Python's standard library.
    Overrides how the error messages are printed.
    """

    def error(self, message):
        """
        Print on **stderr** error message from command line parser and
        exit process.
        """
        self.print_help(sys.stderr)
        self.exit(2, _('\n{:s}: error: {:s}\n').format(self.prog, message))




import datetime as dt

import pytest

from shelter.utils.imports import import_object


def test_import_object():
    assert dt.timedelta == import_object('datetime.timedelta')


@pytest.mark.parametrize('name', ['', 'datetime'])
def test_import_object_fail_when_no_object(name):
    with pytest.raises(ValueError) as e:
        import_object(name)
    assert "Invalid name" in str(e)




from shelter.commands import SHELTER_MANAGEMENT_COMMANDS


def test_shelter_management_commands():
    assert set(SHELTER_MANAGEMENT_COMMANDS) == set([
        'shelter.commands.devserver.DevServer',
        'shelter.commands.runserver.RunServer',
        'shelter.commands.shell.Shell',
        'shelter.commands.showconfig.ShowConfig',
        'shelter.commands.startproject.StartProject',
    ])



"""
Shelter's exceptions.
"""


class ShelterError(Exception):
    """
    Base error, ancestor for all other Shelter's errors.
    """

    pass


class ImproperlyConfiguredError(ShelterError):
    """
    Configuration error.
    """

    pass


class ProcessError(ShelterError):
    """
    Worker/service process error.
    """

    pass


class CommandError(ShelterError):
    """
    Management command error.
    """

    pass



"""
Module :module:`shelter.core.app` provides functions for initializing
Tornado's application.
"""

import tornado.web

from shelter.core.web import NullHandler
from shelter.utils.imports import import_object

__all__ = ['get_tornado_apps']


def get_tornado_apps(context, debug=False):
    """
    Create Tornado's application for all interfaces which are defined
    in the configuration. *context* is instance of the
    :class:`shelter.core.context.Context`. If *debug* is :const:`True`,
    server will be run in **DEBUG** mode. Return :class:`list` of the
    :class:`tornado.web.Application` instances.
    """
    if context.config.app_settings_handler:
        app_settings_handler = import_object(
            context.config.app_settings_handler)
        settings = app_settings_handler(context)
    else:
        settings = {}

    apps = []
    for interface in context.config.interfaces:
        urls = interface.urls
        if not urls:
            urls = [tornado.web.URLSpec('/', NullHandler)]
        apps.append(
            tornado.web.Application(
                urls, debug=debug, context=context,
                interface=interface, **settings
            )
        )
    return apps



"""
Helpers for work with net/sockets.
"""

__all__ = ['parse_host']


def parse_host(host):
    """
    Parse *host* in format ``"[hostname:]port"`` and return :class:`tuple`
    ``(address, port)``.

        >>> parse_host('localhost:4444')
        ('localhost', 4444)
        >>> parse_host(':4444')
        ('', 4444)
        >>> parse_host('4444')
        ('', 4444)
        >>> parse_host('2001:db8::1428:57ab:4444')
        ('2001:db8::1428:57ab', 4444)
        >>> parse_host('localhost')
        ValueError: Invalid port number 'localhost'
    """
    parts = host.split(':')
    address = ':'.join(parts[:-1])
    try:
        port = int(parts[-1])
    except ValueError:
        port = None
    if not port or port < 1 or port > 65535:
        raise ValueError("Invalid port number '%s'" % port)
    return address, port



"""
Module :module:`shelter.core.web` provides an ancestor for HTTP handlers.
"""

import logging

from tornado.web import RequestHandler

__all__ = ['BaseRequestHandler']


class BaseRequestHandler(RequestHandler):
    """
    Base class for handlers, :class:`tornado.web import RequestHandler`
    descendant. Adds *logger* attribute with Python's logger instance,
    *context* with :class:`shelter.core.context.Context` instance and
    *interface* with :class:`shelter.core.config.Config.Interface`
    instance.
    """

    def __init__(self, application, request, **kwargs):
        self.context = application.settings['context']
        self.interface = application.settings['interface']

        super(BaseRequestHandler, self).__init__(
            application, request, **kwargs)

        self.logger = logging.getLogger(
            "{:s}.{:s}".format(__name__, self.__class__.__name__))


class NullHandler(BaseRequestHandler):
    """
    HTTP Handler for '/' when not defined any url to handler route.
    """

    def get(self, *unused_args, **unused_kwargs):
        self.write("Interface '%s' works!\n" % self.interface.name)
        self.set_header("Content-Type", 'text/plain; charset=UTF-8')



"""
Run Python's interactive console.
"""

from shelter.core.commands import BaseCommand


class Shell(BaseCommand):
    """
    Management command which runs interactive Python's shell, either
    **IPython** or **python**.
    """

    name = 'shell'
    help = 'runs a python interactive interpreter'

    def command(self):
        user_ns = {
            'context': self.context,
        }
        try:
            import IPython
            IPython.start_ipython(argv=[], user_ns=user_ns)
        except ImportError:
            import code
            code.interact(local=user_ns)

    def _configure_logging(self):
        """
        Do not configure logging.
        """
        pass




import importlib
import os.path

from shelter.core.cmdlineparser import ArgumentParser

import pytest
import mock

from six.moves import configparser

from shelter.contrib.config.iniconfig import (
    get_conf_d_files, get_conf_files, get_configparser, IniConfig
)
from shelter.core.exceptions import ImproperlyConfiguredError
from shelter.core.context import Context


class ContextTest(Context):
    pass


@pytest.fixture
def test_conf_dir():
    return os.path.join(os.path.dirname(__file__), 'conf')


def test_get_conf_d_files(test_conf_dir):
    conf_d_dir = os.path.join(test_conf_dir, 'example.conf.d')
    expected = [
        os.path.join(conf_d_dir, '10-app.conf'),
        os.path.join(conf_d_dir, '20-database.conf'),
    ]
    assert get_conf_d_files(conf_d_dir) == expected


def test_get_conf_d_files_fail_when_not_a_directory(test_conf_dir):
    with pytest.raises(ValueError) as e:
        get_conf_d_files(os.path.join(test_conf_dir, 'foo'))
    assert 'is not a directory' in str(e)


def test_get_conf_files(test_conf_dir):
    conf_filename = os.path.join(test_conf_dir, 'example.conf')
    conf_d_dir = os.path.join(test_conf_dir, 'example.conf.d')
    expected = [
        conf_filename,
        os.path.join(conf_d_dir, '10-app.conf'),
        os.path.join(conf_d_dir, '20-database.conf'),
    ]
    assert get_conf_files(conf_filename) == expected


def test_get_conf_files_fail_when_not_a_file(test_conf_dir):
    with pytest.raises(ValueError) as e:
        get_conf_files(os.path.join(test_conf_dir, 'foo.conf'))
    assert 'is not a file' in str(e)


@mock.patch('shelter.contrib.config.iniconfig.logger')
def test_get_configparser(m, test_conf_dir):
    conf_filename = os.path.join(test_conf_dir, 'example.conf')
    conf_d_dir = os.path.join(test_conf_dir, 'example.conf.d')
    parser = get_configparser(conf_filename)
    assert isinstance(parser, configparser.RawConfigParser)
    assert parser.get('DEFAULT', 'test_option') == 'bar'
    assert m.mock_calls == [
        mock.call.info("Found config '%s'", conf_filename),
        mock.call.info("Found config '%s'",
                       os.path.join(conf_d_dir, '10-app.conf')),
        mock.call.info("Found config '%s'",
                       os.path.join(conf_d_dir, '20-database.conf')),
    ]


def test_get_configparser_empty_filename():
    with pytest.raises(ImproperlyConfiguredError) as exc_info:
        get_configparser('')
    assert "Configuration file is not defined" in str(exc_info.value)


@mock.patch('shelter.contrib.config.iniconfig.logger')
def test_get_configparser_no_conf_d_dir(m, test_conf_dir):
    conf_filename = os.path.join(test_conf_dir, 'example2.conf')
    parser = get_configparser(conf_filename)
    assert isinstance(parser, configparser.RawConfigParser)
    assert parser.get('DEFAULT', 'test_option') == 'baz'
    expected = [
        mock.call.info("Found config '%s'", conf_filename),
    ]
    assert m.mock_calls == expected


def test_get_configparser_config_does_not_exist(test_conf_dir):
    conf_filename = os.path.join(test_conf_dir, 'example_not_exists.conf')
    with pytest.raises(ValueError) as e:
        get_configparser(conf_filename)
    assert 'is not a file' in str(e)


@mock.patch('shelter.contrib.config.iniconfig.logger')
@mock.patch(
    'shelter.contrib.config.iniconfig.get_conf_d_files', new=lambda x: ['abc'])
def test_get_configparser_warn(m, test_conf_dir):
    conf_filename = os.path.join(test_conf_dir, 'example.conf')
    parser = get_configparser(conf_filename)
    assert isinstance(parser, configparser.RawConfigParser)
    assert parser.get('DEFAULT', 'test_option') == 'baz'
    assert m.mock_calls == [
        mock.call.info("Found config '%s'", conf_filename),
        mock.call.info("Found config '%s'", 'abc'),
        mock.call.warning("Error while parsing config '%s'", 'abc'),
    ]


def test_config_interfaces(test_conf_dir):
    conf_filename = os.path.join(test_conf_dir, 'example.conf')
    parser = ArgumentParser()
    parser.add_argument('-f', dest='config')
    config = IniConfig(importlib.import_module('tests.settings1'),
                       parser.parse_args(['-f', conf_filename]))

    interfaces = sorted(config.interfaces, key=lambda x: x.name)

    assert len(interfaces) == 3

    assert interfaces[0].name == 'fastrpc'
    assert interfaces[0].host == '192.168.1.0'
    assert interfaces[0].port == 4445
    assert interfaces[0].unix_socket is None
    assert len(interfaces[0].urls) == 0

    assert interfaces[1].name == 'http'
    assert interfaces[1].host == ''
    assert interfaces[1].port == 4444
    assert interfaces[1].unix_socket is None
    assert len(interfaces[1].urls) == 2

    assert interfaces[2].name == 'unix'
    assert interfaces[2].host is None
    assert interfaces[2].port is None
    assert interfaces[2].unix_socket == '/tmp/tornadoini.socket'
    assert len(interfaces[2].urls) == 3


def test_config_interfaces_both_tcp_and_unix(test_conf_dir):
    conf_filename = os.path.join(test_conf_dir, 'example5.conf')
    parser = ArgumentParser()
    parser.add_argument('-f', dest='config')
    config = IniConfig(importlib.import_module('tests.settings5'),
                       parser.parse_args(['-f', conf_filename]))

    interface = config.interfaces[0]

    assert interface.name == 'http_both_tcp_and_unix'
    assert interface.host == ''
    assert interface.port == 4444
    assert interface.unix_socket == '/tmp/tornadoini.socket'


def test_config_interfaces_fail_when_neither_tcp_nor_unix(test_conf_dir):
    conf_filename = os.path.join(test_conf_dir, 'example6.conf')
    with pytest.raises(ValueError) as e:
        parser = ArgumentParser()
        parser.add_argument('-f', dest='config')
        config = IniConfig(importlib.import_module('tests.settings6'),
                           parser.parse_args(['-f', conf_filename]))
        _ = config.interfaces
    assert "Interface MUST listen either on TCP or UNIX socket" in str(e)



"""
Helpers for importing Python's modules.
"""

import importlib

__all__ = ['import_object']


def import_object(name):
    """
    Import module and return object from it. *name* is :class:`str` in
    format ``module.path.ObjectClass``.

    ::
        >>> import_command('module.path.ObjectClass')
        <class 'module.path.ObjectClass'>
    """
    parts = name.split('.')
    if len(parts) < 2:
        raise ValueError("Invalid name '%s'" % name)
    module_name = ".".join(parts[:-1])
    obj_name = parts[-1]
    module = importlib.import_module(module_name)
    return getattr(module, obj_name)




import mock

import tornado.web

from shelter.core.config import Config
from shelter.core.web import BaseRequestHandler, NullHandler


def test_base_request_handler():
    app = tornado.web.Application([], context=1, interface=2)
    req = mock.Mock()

    h = BaseRequestHandler(app, req)

    assert h.context == 1
    assert h.interface == 2


def test_null_handler():
    app = tornado.web.Application(
        [], context=object(), interface=Config.Interface(
            name='foo', host='', port=1, unix_socket=None,
            processes=0, urls=[]
        )
    )
    req = mock.Mock()

    with mock.patch.object(NullHandler, 'write') as m_write:
        with mock.patch.object(NullHandler, 'set_header') as m_set_header:
            h = NullHandler(app, req)
            h.get()

    m_write.assert_called_once_with("Interface 'foo' works!\n")
    m_set_header.assert_called_once_with(
        "Content-Type", "text/plain; charset=UTF-8")




NAME = 'test-shelter-app'

MANAGEMENT_COMMANDS = (
    'tests.test_main.Cmd',
)

INTERFACES = {
    'http_both_tcp_and_unix': {
        'LISTEN': ':4443',
        'UNIX_SOCKET': '/tmp/tornado.socket',
        'PROCESSES': 12,
        'URLS': 'tests.urls1.urls_http',
    },
}



"""
Module :module:`shelter.core.context` provides base class which
encapsulates data for HTTP handlers.
"""

__all__ = ['Context']


class Context(object):
    """
    Class which encapsulates data (configuration, database connection,
    other resources...) for HTTP handlers.

    .. warning::

       Instance is created before running the server. Do not create
       instance of the shared resources (sockets, files, ...) in
       constructor, it is necessary initialize them lazy!
    """

    def __init__(self, config):
        self._config = config
        self.initialize()

    @classmethod
    def from_config(cls, config):
        """
        According to application's configuration *config* create and
        return new instance of the **Context**.
        """
        return cls(config)

    def initialize(self):
        """
        Initialize instance attributes. This method is called when instance
        is initialized. You can override this method in the subclasses.
        """
        pass

    def initialize_child(self, process_type, **kwargs):
        """
        Initialize instance attributes, it is similar to :meth:`initialize`.
        However, method is called only in the children (workers of the
        Tornado, service processes) when child is initialized and before
        HTTP server is started. *process_type* indicates type of the process,
        it can be :data:`shelter.core.constants.SERVICE_PROCESS` or
        :data:`shelter.core.constants.TORNADO_WORKER`. *kwargs* contains
        additional data according to *process_type*. You can override this
        method in the subclasses.
        """
        pass

    @property
    def config(self):
        """
        Application's configuration.
        """
        return self._config



"""
Module :module:`shelter.core.constants` contains useful constants used
in Shelter.
"""

__all__ = ['SERVICE_PROCESS', 'TORNADO_WORKER']

SERVICE_PROCESS = 'service_process'
"""
Indicates that type of the process is a service process. *kwargs* argument
in method :meth:`shelter.core.context.initialize_child` contains *process*,
which holds instance of the service process.
"""

TORNADO_WORKER = 'tornado_worker'
"""
Indicates that type of the process is a Tornado HTTP worker. *kwargs*
argument in method :meth:`shelter.core.context.initialize_child` contains
*app*, which holds Tornado's application associated with this worker and
*http_server*, which holds instance of the ``tornado.httpserver.HTTPServer``.
"""




import pytest

from shelter.utils.net import parse_host


@pytest.mark.parametrize(
    'host,expected',
    [
        ('4444', ('', 4444)),
        (':4444', ('', 4444)),
        ('127.0.0.1:4444', ('127.0.0.1', 4444)),
        ('localhost:4444', ('localhost', 4444)),
        ('localhost.localdomain:4444', ('localhost.localdomain', 4444)),
        (
            '2001:0db8:0000:0000:0000:0000:1428:57ab:4444',
            ('2001:0db8:0000:0000:0000:0000:1428:57ab', 4444)
        ),
        ('2001:db8::1428:57ab:4444', ('2001:db8::1428:57ab', 4444)),
        ('::1:4444', ('::1', 4444)),
    ]
)
def test_parse_host(host, expected):
    assert parse_host(host) == expected


@pytest.mark.parametrize(
    'host',
    [
        'localhost:port',
        'localhost:-1',
        'localhost:0',
        'localhost:65536'
    ]
)
def test_parse_host_fail_when_invalid_port_number(host):
    with pytest.raises(ValueError) as e:
        parse_host(host)
    assert "Invalid port number" in str(e)



"""
Module :module:`shelter.contrib.config.iniconfig` provides **INI files**
configuration.
"""

import glob
import logging.config
import os.path

from gettext import gettext as _

import six.moves

from shelter.core.exceptions import ImproperlyConfiguredError
from shelter.core.config import Config, argument
from shelter.utils.imports import import_object
from shelter.utils.net import parse_host

__all__ = ['IniConfig']

logger = logging.getLogger(__name__)

CONFIGPARSER_EXC = (
    six.moves.configparser.NoSectionError,
    six.moves.configparser.NoOptionError,
)


def get_conf_d_files(path):
    """
    Return alphabetical ordered :class:`list` of the *.conf* files
    placed in the path. *path* is a directory path.

    ::

        >>> get_conf_d_files('conf/conf.d/')
        ['conf/conf.d/10-base.conf', 'conf/conf.d/99-dev.conf']
    """
    if not os.path.isdir(path):
        raise ValueError("'%s' is not a directory" % path)
    files_mask = os.path.join(path, "*.conf")
    return [f for f in sorted(glob.glob(files_mask)) if os.path.isfile(f)]


def get_conf_files(filename):
    """
    Return :class:`list` of the all configuration files. *filename* is a
    path of the main configuration file.

    ::

        >>> get_conf_files('exampleapp.conf')
        ['exampleapp.conf', 'exampleapp.conf.d/10-database.conf']
    """
    if not os.path.isfile(filename):
        raise ValueError("'%s' is not a file" % filename)
    conf_d_path = "%s.d" % filename
    if not os.path.exists(conf_d_path):
        return [filename]
    return [filename] + get_conf_d_files(conf_d_path)


def get_configparser(filename=''):
    """
    Read main configuration file and all files from *conf.d* subdirectory
    and return parsed configuration as a **configparser.RawConfigParser**
    instance.
    """
    filename = filename or os.environ.get('SHELTER_CONFIG_FILENAME', '')
    if not filename:
        raise ImproperlyConfiguredError(_(
            "Configuration file is not defined. You must either "
            "set 'SHELTER_CONFIG_FILENAME' environment variable or "
            "'-f/--config-file' command line argument."
        ))

    parser = six.moves.configparser.RawConfigParser()
    for conf_file in get_conf_files(filename):
        logger.info("Found config '%s'", conf_file)
        if not parser.read(conf_file):
            logger.warning("Error while parsing config '%s'", conf_file)
    return parser


class IniConfig(Config):
    """
    Class which extends base :class:`shelter.core.config.Config`. Provides
    configuration from **INI** files. Configuration file is specified
    either by 'SHELTER_CONFIG_FILENAME' environment variable or
    '-f/--config-file' command line argument.

    First, main configuration file is read. Then all configuration files
    from `file.conf.d` subdirectory are read in alphabetical order. E.g.
    if `-f conf/myapp.conf` is handled, first `conf/myapp.conf` file is
    read and then all `conf/myapp.conf.d/*.conf` files. Value in later
    configuration file overrides previous defined value.
    """

    arguments = (
        argument(
            '-f', '--config-file',
            dest='config', action='store', type=str, default=None,
            help=_('configuration file')
        ),
    )

    def __init__(self, settings, args_parser):
        self._config_parser = get_configparser(args_parser.config)
        super(IniConfig, self).__init__(settings, args_parser)

    def configure_logging(self):
        """
        Configure Python's logging according to configuration placed in
        configuration file.
        """
        logging.config.fileConfig(
            self.config_parser, disable_existing_loggers=False)

    @property
    def config_parser(self):
        """
        Parsed configuration file as a **configparser.RawConfigParser**
        instance.
        """
        return self._config_parser

    @property
    def name(self):
        """
        Application name. It's used as a process name.
        """
        try:
            return self.config_parser.get('application', 'name')
        except CONFIGPARSER_EXC:
            return super(IniConfig, self).name

    @property
    def interfaces(self):
        """
        Interfaces as a :class:`list`of the
        :class:`shelter.core.config.Config.Interface` instances.
        """
        if 'interfaces' not in self._cached_values:
            self._cached_values['interfaces'] = []
            for name, interface in six.iteritems(self.settings.INTERFACES):
                interface_name = 'interface_%s' % name
                # Hostname:port + unix socket
                try:
                    listen = self.config_parser.get(interface_name, 'Listen')
                except CONFIGPARSER_EXC:
                    listen = interface.get('LISTEN')
                try:
                    unix_socket = self.config_parser.get(
                        interface_name, 'UnixSocket')
                except CONFIGPARSER_EXC:
                    unix_socket = interface.get('UNIX_SOCKET')
                if not listen and not unix_socket:
                    raise ValueError(
                        'Interface MUST listen either on TCP '
                        'or UNIX socket or both')
                host, port = parse_host(listen) if listen else (None, None)
                # Processes
                try:
                    processes = self.config_parser.getint(
                        interface_name, 'Processes')
                except CONFIGPARSER_EXC:
                    processes = int(interface.get('PROCESSES', 1))
                # Urls
                try:
                    urls_obj_name = self.config_parser.get(
                        interface_name, 'Urls')
                except CONFIGPARSER_EXC:
                    urls_obj_name = interface.get('URLS', '')
                if urls_obj_name:
                    urls = import_object(urls_obj_name)
                else:
                    urls = ()

                self._cached_values['interfaces'].append(
                    self.Interface(
                        name, host, port, unix_socket, processes, urls)
                )
        return self._cached_values['interfaces']




import logging
import os


logger = logging.getLogger(__name__)


def init_handler(context):
    logger.info("'%s' app init handler", context.config.name)


def sigusr1_handler(context):
    logger.info(
        "'%s' app sigusr1 handler, pid %d",
        context.config.name, os.getpid()
    )


def sigusr2_handler(context):
    logger.info(
        "'%s' app sigusr2 handler, pid %d",
        context.config.name, os.getpid()
    )


def app_settings_handler(context):
    return {
        'cookie_secret': context.config.secret_key,
    }




from tornado.web import URLSpec

from shelter.core.web import NullHandler


urls_http = (
    URLSpec('/baz/', NullHandler),
    URLSpec('/bar/', NullHandler),
)


urls_unix = (
    URLSpec('/baz/', NullHandler),
    URLSpec('/bar/', NullHandler),
    URLSpec('/foo/', NullHandler),
)




import mock
import pytest

import shelter.main

from shelter.core.cmdlineparser import argument
from shelter.core.commands import BaseCommand
from shelter.commands.startproject import StartProject
from shelter.core.config import Config


class Cmd(BaseCommand):

    name = 'testcmd'
    help = 'test cmd help text'
    arguments = (
        argument('--name', dest='name', help='your name'),
    )


class CmdBadType(object):
    pass


class AppConfig(Config):
    pass


class AppConfigBadType(object):
    pass


@mock.patch.object(BaseCommand, 'command')
def test_main(m):
    with pytest.raises(SystemExit) as e:
        shelter.main.main(['-s', 'tests.settings1', 'testcmd'])
    m.assert_called_once_with()
    assert e.value.code == 0


def test_main_exit_when_no_settings_module():
    with pytest.raises(SystemExit) as e:
        shelter.main.main(['shell'])
    assert e.value.code == 2


def test_main_fail_when_settings_does_not_exist():
    with pytest.raises(SystemExit) as e:
        shelter.main.main(['-s', 'tests.settings', 'testcmd'])
    assert e.value.code == 2


def test_main_default_config_class():

    def command(self):
        assert type(self.context.config) == Config

    with mock.patch.object(BaseCommand, 'command', new=command):
        with pytest.raises(SystemExit) as e:
            shelter.main.main(['-s', 'tests.settings1', 'testcmd'])
        assert e.value.code == 0


def test_main_default_config_class_when_settings_is_not_required():

    def command(self):
        assert type(self.context.config) == Config

    with mock.patch.object(StartProject, 'command', new=command):
        with pytest.raises(SystemExit) as e:
            shelter.main.main(['startproject', 'foo'])
        assert e.value.code == 0


def test_main_custom_config_class():

    def command(self):
        assert type(self.context.config) == AppConfig

    with mock.patch.object(BaseCommand, 'command', new=command):
        with pytest.raises(SystemExit) as e:
            shelter.main.main(['-s', 'tests.settings3', 'testcmd'])
        assert e.value.code == 0


def test_main_custom_config_class_bad_type():
    with pytest.raises(TypeError) as e:
        shelter.main.main(['-s', 'tests.settings4', 'testcmd'])
    assert "must be subclass of the shelter.core.config.Config" in str(e)


def test_main_fail_when_invalid_command_subclass():
    with pytest.raises(ValueError) as e:
        shelter.main.main(['-s', 'tests.settings2', 'testcmd'])
    assert "is not subclass of the BaseCommand" in str(e)


def test_main_exit_when_no_action():
    with pytest.raises(SystemExit) as e:
        shelter.main.main(['-s', 'tests.settings1'])
    assert e.value.code == 2


@mock.patch.object(BaseCommand, 'command')
def test_main_fail_when_command_error(m):
    m.side_effect = [KeyError]
    with pytest.raises(SystemExit) as e:
        shelter.main.main(['-s', 'tests.settings1', 'testcmd'])
    m.assert_called_once_with()
    assert e.value.code == 1


@mock.patch.object(BaseCommand, 'command')
def test_main_force_exit_when_children(m_command):
    m_command.side_effect = [KeyError]
    with mock.patch('shelter.main.multiprocessing') as m_multiprocessing:
        m_multiprocessing.active_children.return_value = [mock.Mock()]
        with mock.patch('os._exit') as m_os_exit:
            m_os_exit.side_effect = [SystemExit]
            with pytest.raises(SystemExit):
                shelter.main.main(['-s', 'tests.settings1', 'testcmd'])
    m_os_exit.assert_called_once_with(1)




from shelter.core.cmdlineparser import argument


def test_argument_empty():
    assert argument() == ((), {})


def test_argument_args():
    assert argument('-h', '--help') == (('-h', '--help'), {})


def test_argument_kwargs():
    expected = ((), {'dest': 'help', 'type': str})
    assert argument(dest='help', type=str) == expected


def test_argument_both():
    expected = (('-h', '--help'), {'dest': 'help', 'type': str})
    assert argument('-h', '--help', dest='help', type=str) == expected




from tornado.web import URLSpec

from shelter.core.web import NullHandler

from exampleapp.handlers import ShowValueHandler


urls_default = (
    URLSpec(r'/', NullHandler),
    URLSpec(r'/value/', ShowValueHandler),
)



"""
Run development server.
"""

import signal

import tornado.ioloop
import tornado.netutil
import tornado.httpserver

from shelter.core.app import get_tornado_apps
from shelter.core.commands import BaseCommand


class DevServer(BaseCommand):
    """
    Management command which runs development server. Server is run in
    DEBUG mode.
    """

    name = 'devserver'
    help = 'run server for local development'

    def command(self):
        # For each interface create Tornado application and start
        # listening on the port.
        listen_on = []
        for app in get_tornado_apps(self.context, debug=True):
            interface = app.settings['interface']

            if not interface.port and not interface.unix_socket:
                raise ValueError(
                    'Interface MUST listen either on TCP '
                    'or UNIX socket or both')

            server = tornado.httpserver.HTTPServer(app)

            sockets = []
            if interface.port:
                host, port = interface.host, interface.port
                sockets.extend(
                    tornado.netutil.bind_sockets(port, address=host))
                listen_on.append(
                    "{:s}:{:d}".format(host, port))
            if interface.unix_socket:
                sockets.append(
                    tornado.netutil.bind_unix_socket(interface.unix_socket))
                listen_on.append(
                    "{:s}".format(interface.unix_socket))
            server.add_sockets(sockets)

        # Run IOLoop
        if listen_on:
            self.stdout.write(
                "Start dev server on %s, press Ctrl+C to stop\n" %
                ", ".join(listen_on)
            )
            self.stdout.flush()

            try:
                def signal_handler(unused_signum, unused_frame):
                    """
                    Handle SIGTERM signal. Stop IOLoop and exit.
                    """
                    tornado.ioloop.IOLoop.instance().add_callback(
                        tornado.ioloop.IOLoop.instance().stop)
                signal.signal(signal.SIGTERM, signal_handler)

                tornado.ioloop.IOLoop.instance().start()
            except KeyboardInterrupt:
                tornado.ioloop.IOLoop.instance().add_callback(
                    tornado.ioloop.IOLoop.instance().stop)




NAME = 'exampleapp'

# Change it!!!
SECRET_KEY = '!VstwEzRZ3Stqd5#3V$tAA#cDbedg@!rVz!xw!R5xBZS2B#bbE@DdZe#DF5s!@zv'

INIT_HANDLER = 'exampleapp.app.init_handler'

SIGUSR1_HANDLER = 'exampleapp.app.sigusr1_handler'

SIGUSR2_HANDLER = 'exampleapp.app.sigusr2_handler'

APP_SETTINGS_HANDLER = 'exampleapp.app.app_settings_handler'

CONFIG_CLASS = 'exampleapp.config.Config'

CONTEXT_CLASS = 'exampleapp.context.Context'

MANAGEMENT_COMMANDS = (
    'exampleapp.commands.HelloWorld',
)

SERVICE_PROCESSES = (
    ('exampleapp.processes.WarmCache', True, 5),
)

INTERFACES = {
    'default': {
        'LISTEN': ':8000',
        'PROCESSES': 2,
        'URLS': 'exampleapp.urls.urls_default',
    },
    'debug': {
        'LISTEN': ':8001',
        'PROCESSES': 1,
    },
}




import os.path

import shelter.main


def main():
    os.environ['SHELTER_SETTINGS_MODULE'] = 'exampleapp.settings'
    os.environ['SHELTER_CONFIG_FILENAME'] = os.path.join(
        os.path.dirname(__file__), os.pardir, 'conf', 'exampleapp.conf')
    shelter.main.main()



"""
Run production server.
"""

import ctypes
import logging
import multiprocessing
import os
import signal
import sys
import time

import setproctitle
import six

import tornado.httpserver
import tornado.ioloop
import tornado.netutil
import tornado.process

from shelter.core.app import get_tornado_apps
from shelter.core.commands import BaseCommand
from shelter.core.constants import TORNADO_WORKER
from shelter.core.exceptions import ProcessError
from shelter.utils.imports import import_object

SIGNALS_TO_NAMES_DICT = {
    getattr(signal, n): n for n in dir(signal) if
    n.startswith('SIG') and '_' not in n
}


class ProcessWrapper(object):
    """
    Container for process, provides API for controlling processes.
    *process_cls* is class of the process, either service process
    or Tornado's HTTP server process. *args* are arguments which
    are handled into process constructor. If *wait_unless_ready* is
    :data:`True`, wait maximum *timeout* seconds unless process is
    started, otherwise raise :exc:`shelter.core.exceptions.ProcessError`
    exception if time is exceeded. If *timeout* is :data:`None`,
    wait infinity seconds.
    """

    def __init__(
            self, process_cls, process_args, wait_unless_ready=True,
            timeout=None, name=None):

        self._process = None
        self._process_cls = process_cls
        self._process_args = process_args
        self._wait_unless_ready = wait_unless_ready
        self._timeout = timeout

        self.name = name if name is not None else process_cls.__name__

    if six.PY3:
        def __bool__(self):
            return self.is_alive
    else:
        def __nonzero__(self):
            return self.is_alive

    @property
    def pid(self):
        """
        Either **PID** of the process or :data:`None` if process has
        not been run yet.
        """
        if self._process is None:
            return None
        return self._process.pid

    @property
    def exitcode(self):
        """
        Process exit code. :const:`0` when process exited successfully,
        positive number when exception was occurred, negative number when
        process was signaled and :data:`None` when process has not exited
        yet.
        """
        if self._process is None:
            raise ProcessError(
                "Process '%s' has not been started yet" % self.name)
        return self._process.exitcode

    @property
    def has_started(self):
        """
        :data:`True` when process has been started, else :data:`False`.
        """
        return self._process is not None

    @property
    def is_alive(self):
        """
        :const:`True` when process has been run and has not exited yet,
        else :const:`False`.
        """
        if self._process is None:
            return False
        return self._process.is_alive()

    def start(self):
        """
        Run the process.
        """
        if self:
            raise ProcessError(
                "Process '%s' has been already started" % self.name)
        first_run = not self.has_started
        # Run process
        self._process = self._process_cls(*self._process_args)
        self._process.daemon = False
        self._process.start()
        # Wait unless process is successfully started
        if first_run and self._wait_unless_ready:
            if self._timeout:
                stop_time = time.time() + self._timeout
                while time.time() < stop_time and not self._process.ready:
                    time.sleep(0.25)
                if not self._process.ready:
                    raise ProcessError(
                        "Timeout during start process '%s'" % self.name)
            else:
                while not self._process.ready:
                    time.sleep(0.25)

    def stop(self):
        """
        Stop the process.
        """
        if self:
            self._process.stop()


class TornadoProcess(multiprocessing.Process):
    """
    Tornado HTTP worker.
    """

    def __init__(self, tornado_app, sockets):
        super(TornadoProcess, self).__init__()

        self._http_server = None
        self._parent_pid = os.getpid()
        self._ready = multiprocessing.Value(ctypes.c_bool, False)
        self._sockets = sockets
        self._tornado_app = tornado_app

        self.context = self._tornado_app.settings['context']
        self.http_server = None
        self.logger = logging.getLogger(
            "{:s}.{:s}".format(__name__, self.__class__.__name__))

    @property
    def ready(self):
        """
        :data:`True` when HTTP worker has been started successfully,
        else :data:`False`.
        """
        return self._ready.value

    def stop(self):
        """
        Stop the worker.
        """
        if self._http_server is not None:
            self._http_server.stop()
        tornado.ioloop.IOLoop.instance().add_callback(
            tornado.ioloop.IOLoop.instance().stop)

    def run(self):
        """
        Tornado worker which handles HTTP requests.
        """
        setproctitle.setproctitle("{:s}: worker {:s}".format(
            self.context.config.name,
            self._tornado_app.settings['interface'].name))
        self.logger.info(
            "Worker '%s' has been started with pid %d",
            self._tornado_app.settings['interface'].name, os.getpid())

        # Configure logging
        self.context.config.configure_logging()
        # Create HTTP server instance
        self.http_server = tornado.httpserver.HTTPServer(self._tornado_app)
        # Initialize child
        self.context.initialize_child(TORNADO_WORKER, process=self)

        # Register SIGINT handler which will stop worker
        def sigint_handler(unused_signum, unused_frame):
            """
            Call :meth:`stop` method when SIGINT is reached.
            """
            io_loop = tornado.ioloop.IOLoop.instance()
            io_loop.add_callback_from_signal(self.stop)
        signal.signal(signal.SIGINT, sigint_handler)

        # Register callback which is called when IOLoop is started
        def run_ioloop_callback():
            """
            Set ready flag. Callback is called when worker is started.
            """
            self._ready.value = True
        tornado.ioloop.IOLoop.instance().add_callback(run_ioloop_callback)

        # Register job which will stop worker if parent process PID is changed
        def check_parent_callback():
            """
            Tornado's callback function which checks PID of the parent
            process. If PID of the parent process is changed (parent has
            stopped), call :meth:`stop` method.
            """
            if os.getppid() != self._parent_pid:
                self.stop()
        stop_callback = tornado.ioloop.PeriodicCallback(
            check_parent_callback, 250)
        stop_callback.start()

        # Run HTTP server
        self.http_server.add_sockets(self._sockets)
        # Run IOLoop
        tornado.ioloop.IOLoop.instance().start()


class RunServer(BaseCommand):
    """
    Management command which runs production server.
    """

    name = 'runserver'
    help = 'run server'

    main_pid = None
    processes = []

    def initialize(self):
        """
        Initialize instance attributes. You can override this method in
        the subclasses.
        """
        self.main_pid = os.getpid()
        self.processes.extend(self.init_service_processes())
        self.processes.extend(self.init_tornado_workers())

    def sigusr1_handler(self, unused_signum, unused_frame):
        """
        Handle SIGUSR1 signal. Call function which is defined in the
        **settings.SIGUSR1_HANDLER**. If main process, forward the
        signal to all child processes.
        """
        for process in self.processes:
            if process.pid and os.getpid() == self.main_pid:
                try:
                    os.kill(process.pid, signal.SIGUSR1)
                except ProcessLookupError:
                    pass
        if self._sigusr1_handler_func is not None:
            self._sigusr1_handler_func(self.context)

    def init_service_processes(self):
        """
        Prepare processes defined in the **settings.SERVICE_PROCESSES**.
        Return :class:`list` of the :class:`ProcessWrapper` instances.
        """
        processes = []

        for process_struct in getattr(
                self.context.config.settings, 'SERVICE_PROCESSES', ()):
            process_cls = import_object(process_struct[0])
            wait_unless_ready, timeout = process_struct[1], process_struct[2]

            self.logger.info("Init service process '%s'", process_cls.__name__)

            processes.append(
                ProcessWrapper(
                    process_cls, (self.context,),
                    wait_unless_ready=wait_unless_ready,
                    timeout=timeout
                )
            )

        return processes

    def init_tornado_workers(self):
        """
        Prepare worker instances for all Tornado applications. Return
        :class:`list` of the :class:`ProcessWrapper` instances.
        """
        workers = []

        for tornado_app in get_tornado_apps(self.context, debug=False):
            interface = tornado_app.settings['interface']

            if not interface.port and not interface.unix_socket:
                raise ValueError(
                    'Interface MUST listen either on TCP '
                    'or UNIX socket or both')

            name, processes, host, port, unix_socket = (
                interface.name, interface.processes,
                interface.host, interface.port, interface.unix_socket)
            if processes <= 0:
                processes = tornado.process.cpu_count()
            sockets = []
            listen_on = []

            if port:
                sockets.extend(tornado.netutil.bind_sockets(port, host))
                listen_on.append("{:s}:{:d}".format(host, port))
            if unix_socket:
                sockets.append(tornado.netutil.bind_unix_socket(unix_socket))
                listen_on.append("{:s}".format(interface.unix_socket))

            self.logger.info(
                "Init %d worker(s) for interface '%s' (%s)",
                processes, name, ", ".join(listen_on))

            for dummy_i in six.moves.range(processes):
                worker = ProcessWrapper(
                    TornadoProcess, (tornado_app, sockets),
                    wait_unless_ready=True, timeout=5.0,
                    name=name
                )
                workers.append(worker)

        return workers

    def start_processes(self, max_restarts=-1):
        """
        Start processes and check their status. When some process crashes,
        start it again. *max_restarts* is maximum amount of the restarts
        across all processes. *processes* is a :class:`list` of the
        :class:`ProcessWrapper` instances.
        """
        while 1:
            for process in self.processes:
                if not process:
                    # When process has not been started, start it
                    if not process.has_started:
                        process.start()
                        continue
                    # When process has stopped, start it again
                    exitcode = process.exitcode
                    if exitcode != 0:
                        # Process has been signaled or crashed
                        if exitcode > 0:
                            self.logger.error(
                                "Process '%s' with pid %d died with exitcode "
                                "%d", process.name, process.pid, exitcode
                            )
                        else:
                            self.logger.error(
                                "Process '%s' with pid %d died due to %s",
                                process.name, process.pid,
                                SIGNALS_TO_NAMES_DICT[abs(exitcode)]
                            )
                        # Max restarts has been reached, exit
                        if not max_restarts:
                            self.logger.fatal("Too many child restarts")
                            break
                        # Start process again
                        process.start()
                        # Decrement max_restarts counter
                        if max_restarts > 0:
                            max_restarts -= 1
                    else:
                        # Process has stopped without error
                        self.logger.info(
                            "Process '%s' with pid %d has stopped",
                            process.name, process.pid
                        )
                        # Start process again
                        process.start()
                        self.logger.info(
                            "Process '%s' has been started with pid %d",
                            process.name, process.pid
                        )
            else:
                time.sleep(0.25)
                continue
            break

    def command(self):
        """
        **runserver** command implementation.
        """
        setproctitle.setproctitle(
            "{:s}: master process '{:s}'".format(
                self.context.config.name, " ".join(sys.argv)
            ))

        # Init and start processes
        try:
            self.start_processes(max_restarts=100)
        except KeyboardInterrupt:
            pass
        # Stop processes
        for process in self.processes:
            process.stop()



#!/usr/bin/env python

from setuptools import setup


setup(
    name="exampleapp",
    version='0.0.0',
    author='Seznam.cz, a.s.',
    author_email="jan.seifert@firma.seznam.cz",
    description=(
        "Example application which use Shelter."
    ),
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    packages=['exampleapp'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'shelter',
        'six',
        'setproctitle<1.2',
    ],
    entry_points={
        'console_scripts': [
            'manage-exampleapp = exampleapp:main',
        ]
    },
)



"""
Modul :module:`shelter.main` is an entry point from command line into
**Shelter** application.
"""

import importlib
import itertools
import multiprocessing
import os
import sys
import traceback

from gettext import gettext as _

import six

from shelter.commands import SHELTER_MANAGEMENT_COMMANDS
from shelter.core.exceptions import ImproperlyConfiguredError
from shelter.core.commands import BaseCommand
from shelter.core.cmdlineparser import ArgumentParser
from shelter.core.config import Config
from shelter.utils.imports import import_object


__all__ = ['main']


def get_app_settings(parser, known_args):
    """
    Return **settings** module of the application according to
    either command line argument or **SHELTER_SETTINGS_MODULE**
    environment variable.
    """
    args, dummy_remaining = parser.parse_known_args(known_args)
    settings_module_path = (
        args.settings or os.environ.get('SHELTER_SETTINGS_MODULE', ''))
    if not settings_module_path:
        return None
    return importlib.import_module(settings_module_path)


def get_management_commands(settings):
    """
    Find registered managemend commands and return their classes
    as a :class:`dict`. Keys are names of the management command
    and values are classes of the management command.
    """
    app_commands = getattr(settings, 'MANAGEMENT_COMMANDS', ())
    commands = {}
    for name in itertools.chain(SHELTER_MANAGEMENT_COMMANDS, app_commands):
        command_obj = import_object(name)
        if not issubclass(command_obj, BaseCommand):
            raise ValueError("'%s' is not subclass of the BaseCommand" % name)
        commands[command_obj.name] = command_obj
    return commands


def get_config_class(settings):
    """
    According to **settings.CONFIG_CLASS** return either config class
    defined by user or default :class:`shelter.core.config.Config`.
    """
    config_cls_name = getattr(settings, 'CONFIG_CLASS', '')
    if config_cls_name:
        config_cls = import_object(config_cls_name)
    else:
        config_cls = Config
    return config_cls


def main(args=None):
    """
    Run management command handled from command line.
    """
    # Base command line parser. Help is not allowed because command
    # line is parsed in two stages - in the first stage is found setting
    # module of the application, in the second stage are found management
    # command's arguments.
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        '-s', '--settings',
        dest='settings', action='store', type=str, default=None,
        help=_('application settings module')
    )

    # Get settings module
    try:
        settings = get_app_settings(parser, args)
    except ImportError as exc:
        parser.error(_("Invalid application settings module: {}").format(exc))

    # Get management commands and add their arguments into command
    # line parser
    commands = get_management_commands(settings)
    subparsers = parser.add_subparsers(
        dest='action', help=_('specify action')
    )
    for command_cls in six.itervalues(commands):
        subparser = subparsers.add_parser(
            command_cls.name, help=command_cls.help)
        for command_args, kwargs in command_cls.arguments:
            subparser.add_argument(*command_args, **kwargs)

    # Get config class and add its arguments into command line parser
    if settings:
        config_cls = get_config_class(settings)
        if not issubclass(config_cls, Config):
            raise TypeError(
                "Config class must be subclass of the "
                "shelter.core.config.Config")
        for config_args, kwargs in config_cls.arguments:
            parser.add_argument(*config_args, **kwargs)
    else:
        config_cls = Config

    # Add help argument and parse command line
    parser.add_argument(
        '-h', '--help', action='help',
        help=_('show this help message and exit')
    )
    cmdline_args = parser.parse_args(args)
    if not cmdline_args.action:
        parser.error(_('No action'))

    # Run management command
    command_cls = commands[cmdline_args.action]
    if command_cls.settings_required and not settings:
        parser.error(_(
            "Settings module is not defined. You must either set "
            "'SHELTER_SETTINGS_MODULE' environment variable or "
            "'-s/--settings' command line argument."
        ))
    try:
        config = config_cls(settings, cmdline_args)
    except ImproperlyConfiguredError as exc:
        parser.error(str(exc))
    command = command_cls(config)
    try:
        command()
    except Exception:
        traceback.print_exc(file=sys.stderr)
        sys.stderr.flush()
        if multiprocessing.active_children():
            # If main process has children processes, exit immediately without
            # cleaning. It is a workaround, because parent process waits for
            # non-daemon children.
            os._exit(1)
        sys.exit(1)
    sys.exit(0)


if __name__ == '__main__':
    main()




import ctypes
import multiprocessing

from shelter.core import context


class Context(context.Context):

    def __init__(self, config):
        self._value = multiprocessing.Array(ctypes.c_char, 8)
        super(Context, self).__init__(config)

    @property
    def value(self):
        return self._value.value

    @value.setter
    def value(self, v):
        self._value.value = v



"""
Create new application skeleton.
"""

import glob
import os.path

from gettext import gettext as _
from string import Template

import six

from shelter.core.commands import BaseCommand, argument
from shelter.core.exceptions import CommandError


def substitute(template, mapping=None):
    """
    Render the template *template*. *mapping* is a :class:`dict` with
    values to add to the template.
    """
    if mapping is None:
        mapping = {}
    templ = Template(template)
    return templ.substitute(mapping)


def dirtree(path):
    """
    Find recursively and return all files and directories from
    the path *path*.
    """
    results = []
    for name in glob.glob(os.path.join(path, '*')):
        results.append(name)
        if os.path.isdir(name):
            results.extend(dirtree(name))
    return results


class StartProject(BaseCommand):
    """
    Management command which generates new application skeleton.
    """

    name = 'startproject'
    help = _('generate new application skeleton')
    arguments = (
        argument(
            'name',
            type=str, nargs=1,
            help=_('new project name')
        ),
        argument(
            '--author-name',
            type=str, dest='author_name', default='Your Name',
            help=_("author's name")
        ),
        argument(
            '--author-email',
            type=str, dest='author_email', default='your.email@example.com',
            help=_("author's e-mail")
        ),
    )
    settings_required = False

    def command(self):
        substitute_map = {
            'python': 'python3' if six.PY3 else 'python2',
            'package': self.context.config.args_parser.name[0],
            'author_name': self.context.config.args_parser.author_name,
            'author_email': self.context.config.args_parser.author_email,
        }

        skel_dir = os.path.join(os.path.dirname(__file__), 'startproject_skel')
        dest_dir = os.path.join(os.getcwd(), substitute_map['package'])

        if os.path.isdir(dest_dir):
            raise CommandError("%s already exists" % substitute_map['package'])
        else:
            os.mkdir(dest_dir)

        # Find all files in the project template
        suffix = '.template'
        suffix_len = len(suffix)
        prefix_len = len(skel_dir) + 1
        for src_name in dirtree(skel_dir):
            # Build destination name
            dst_name = substitute(
                os.path.join(dest_dir, src_name[prefix_len:]),
                substitute_map)
            if dst_name.endswith(suffix):
                dst_name = dst_name[:-suffix_len]
            # Copy source to destination
            if os.path.isdir(src_name) and not os.path.isdir(dst_name):
                # Make directory which doesn't exist
                os.mkdir(dst_name)
            else:
                # Copy file and adjust content
                with open(src_name, 'rt') as f_src,\
                        open(dst_name, 'wt') as f_dst:
                    f_dst.write(substitute(f_src.read(), substitute_map))
            # Keep permissions
            os.chmod(dst_name, os.stat(src_name).st_mode)



"""
Module :module:`shelter.core.commands` provides an ancestor and
functionality for writing management commands.
"""

import logging
import signal
import sys

from shelter.core.cmdlineparser import argument
from shelter.utils.imports import import_object

__all__ = ['BaseCommand', 'argument']


class BaseCommand(object):
    """
    Ancestor for management commands. Inherit this class and adjust
    *name* and *help* and optionally *arguments* attributes and override
    *command()* method. *arguments* is :class:`tuple` containing command
    line arguments for management command. Classes (each class represents
    one management command) are registered in the
    **settings.MANAGEMENT_COMMANDS** option. Constructor's argument
    *config* is instance of the :class:`shelter.core.config.Config`.

    ::

        # application/commands/hello.py

        from shelter.core.commands import BaseCommand, argument

        class HelloCommand(BaseCommand):

            name = 'hello'
            help = 'show hello'
            arguments = (
                argument('--name', dest='name', help='your name'),
            )

            def command(self):
                print("Hello %s" % self.args.name)


        # application/settings.py

        MANAGEMENT_COMMANDS = (
            'application.commands.hello.HelloWorld',
        )
    """

    help = ''
    """
    Description of the management command.
    """

    arguments = ()
    """
    Command line arguments of the management command.
    """

    settings_required = True
    """
    Settings module required flag. If :const:`True`, settings module
    is necessary for start the application.
    """

    def __init__(self, config):
        self._sigusr1_handler_func = None
        self._sigusr2_handler_func = None

        self.context = config.context_class.from_config(config)
        self.logger = logging.getLogger(
            "{:s}.{:s}".format(__name__, self.__class__.__name__))
        self.stdout = sys.stdout
        self.stderr = sys.stderr

    def initialize(self):
        """
        Initialize instance attributes. You can override this method in
        the subclasses.
        """
        pass

    def __call__(self):
        # Initialize logging
        self._configure_logging()

        # Call application init handler(s)
        init_handler_setting = self.context.config.init_handler
        if init_handler_setting:
            if isinstance(init_handler_setting, str):
                init_handlers = [init_handler_setting]
            else:
                init_handlers = init_handler_setting
            for init_handler in init_handlers:
                self.logger.info("Run init handler '%s'", init_handler)
                init_handler_obj = import_object(init_handler)
                init_handler_obj(self.context)

        # Register SIGUSR1 handler
        handler_name = self.context.config.sigusr1_handler
        if handler_name:
            self.logger.info("Register SIGUSR1 handler '%s'", handler_name)
            self._sigusr1_handler_func = import_object(handler_name)
            signal.signal(signal.SIGUSR1, self.sigusr1_handler)
        else:
            signal.signal(signal.SIGUSR1, signal.SIG_IGN)
        # Register SIGUSR2 handler
        handler_name = self.context.config.sigusr2_handler
        if handler_name:
            self.logger.info("Register SIGUSR2 handler '%s'", handler_name)
            self._sigusr2_handler_func = import_object(handler_name)
            signal.signal(signal.SIGUSR2, self.sigusr2_handler)
        else:
            signal.signal(signal.SIGUSR2, signal.SIG_IGN)

        # Call initialization method
        self.initialize()

        # Run command
        self.command()

    def _configure_logging(self):
        """
        Configure logging.
        """
        self.context.config.configure_logging()

    def sigusr1_handler(self, unused_signum, unused_frame):
        """
        Handle SIGUSR1 signal. Call function which is defined in the
        **settings.SIGUSR1_HANDLER**.
        """
        if self._sigusr1_handler_func is not None:
            self._sigusr1_handler_func(self.context)

    def sigusr2_handler(self, unused_signum, unused_frame):
        """
        Handle SIGUSR2 signal. Call function which is defined in the
        **settings.SIGUSR2_HANDLER**.
        """
        if self._sigusr1_handler_func is not None:
            self._sigusr2_handler_func(self.context)

    def command(self):
        """
        Management command, override this method.
        """
        raise NotImplementedError




from shelter.core.commands import BaseCommand


class HelloWorld(BaseCommand):

    name = "hello"
    help = "prints hello world"

    def command(self):
        self.stdout.write("Hello World!\n")
        self.stdout.flush()



"""
Show effective configuration.
"""

from pprint import pformat

from shelter.core.commands import BaseCommand


class ShowConfig(BaseCommand):
    """
    Management command which shows effective configuration.
    """

    name = 'showconfig'
    help = 'show effective configuration'

    def command(self):
        for k, value in self.context.config.get_config_items():
            self.stdout.write("{}: {}\n".format(k, pformat(value)))
            self.stdout.flush()




import importlib

from shelter.core.app import get_tornado_apps
from shelter.core.cmdlineparser import ArgumentParser
from shelter.core.config import Config
from shelter.core.context import Context


def test_get_tornado_apps():
    config = Config(
        importlib.import_module('tests.settings1'),
        ArgumentParser()
    )
    context = config.context_class.from_config(config)

    apps = sorted(
        get_tornado_apps(context), key=lambda x: x.settings['interface'].name)

    assert len(apps) == 3

    assert isinstance(apps[0].settings['context'], Context)
    assert isinstance(apps[0].settings['interface'], Config.Interface)
    assert apps[0].settings['interface'].name == 'fastrpc'

    assert isinstance(apps[1].settings['context'], Context)
    assert isinstance(apps[1].settings['interface'], Config.Interface)
    assert apps[1].settings['interface'].name == 'http'

    assert isinstance(apps[2].settings['context'], Context)
    assert isinstance(apps[2].settings['interface'], Config.Interface)
    assert apps[2].settings['interface'].name == 'unix'




import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

from shelter import __version__ as VERSION


class PyTest(TestCommand):

    user_options = [
        ('pytest-args=', 'a', "Arguments to pass to py.test"),
    ]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


description = (
    "Simple Python's Tornado wrapper which provides helpers for creation"
    "a new project, writing management commands, service processes, ..."
)

try:
    if sys.version_info >= (3,):
        long_description = open('README.rst', 'rb').read().decode('utf-8')
    else:
        long_description = open('README.rst', 'r').read().decode('utf-8')
except IOError:
        long_description = description

setup(
    name="shelter",
    version=VERSION,
    author='Jan Seifert (Seznam.cz, a.s.)',
    author_email="jan.seifert@firma.seznam.cz",
    description=description,
    long_description=long_description,
    license="BSD",
    url='https://github.com/seznam/shelter',
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    platforms=['any'],
    packages=find_packages(include=['shelter*']),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'tornado>=3.2',
        'six',
        'setproctitle>=1.1',
    ],
    tests_require=[
        'pytest-cov',
        'pytest',
        'mock',
    ],
    test_suite='tests',
    cmdclass={
        'test': PyTest,
    },
    entry_points={
        'console_scripts': [
            'shelter-admin = shelter.main:main',
        ]
    },
)




import logging

import mock
import pytest

from shelter.core.commands import BaseCommand


def test_base_command():
    context = mock.Mock()
    context.config.settings.SERVICE_PROCESSES = ()
    context.config.logging = {}
    context.config.name = 'test-shelter-app'
    context.config.init_handler = None
    context.config.sigusr1_handler = None
    context.config.sigusr2_handler = None
    context.config.logging_from_config_file = False
    config = mock.Mock()
    config.context_class.from_config.return_value = context

    cmd = BaseCommand(config)
    with pytest.raises(NotImplementedError):
        cmd()

    assert cmd.context == context
    assert isinstance(cmd.logger, logging.Logger)
    assert cmd.logger.name == 'shelter.core.commands.BaseCommand'




import importlib

import pytest

from shelter.core.cmdlineparser import ArgumentParser
from shelter.core.config import Config
from shelter.core.context import Context


class ContextTest(Context):
    pass


def test_config_cls():
    config = Config(1, 2)
    assert "<shelter.core.config.Config: 0x" in repr(config)
    assert config.settings == 1
    assert config.args_parser == 2


def test_config_context_class_default():
    config = Config(
        importlib.import_module('tests.settings1'),
        ArgumentParser()
    )
    assert config.context_class is Context


def test_config_context_class_user():
    config = Config(
        importlib.import_module('tests.settings2'),
        ArgumentParser()
    )
    assert config.context_class is not Context
    assert config.context_class is ContextTest


def test_config_interfaces():
    config = Config(
        importlib.import_module('tests.settings1'),
        ArgumentParser()
    )

    interfaces = sorted(config.interfaces, key=lambda x: x.name)

    assert len(interfaces) == 3

    assert interfaces[0].name == 'fastrpc'
    assert interfaces[0].host == '192.168.1.0'
    assert interfaces[0].port == 4445
    assert len(interfaces[0].urls) == 0

    assert interfaces[1].name == 'http'
    assert interfaces[1].host == ''
    assert interfaces[1].port == 4443
    assert len(interfaces[1].urls) == 2

    assert interfaces[2].name == 'unix'
    assert interfaces[2].host is None
    assert interfaces[2].port is None
    assert interfaces[2].unix_socket == '/tmp/tornado.socket'
    assert len(interfaces[2].urls) == 3


def test_config_interfaces_both_tcp_and_unix():
    config = Config(
        importlib.import_module('tests.settings5'),
        ArgumentParser()
    )

    interface = config.interfaces[0]

    assert interface.name == 'http_both_tcp_and_unix'
    assert interface.host == ''
    assert interface.port == 4443
    assert interface.unix_socket == '/tmp/tornado.socket'


def test_config_interface_fail_when_neither_tcp_nor_unix():
    config = Config(
        importlib.import_module('tests.settings6'),
        ArgumentParser()
    )

    with pytest.raises(ValueError) as e:
        _ = config.interfaces
    assert "Interface MUST listen either on TCP or UNIX socket" in str(e)




import base64
import os

from shelter.core.processes import BaseProcess


class WarmCache(BaseProcess):

    interval = 10

    def loop(self):
        self.context.value = base64.b64encode(os.urandom(6))




NAME = 'test-shelter-app'

MANAGEMENT_COMMANDS = (
    'tests.test_main.Cmd',
)

INTERFACES = {
    'http_neither_tcp_nor_unix': {
        'PROCESSES': 12,
        'URLS': 'tests.urls1.urls_http',
    },
}




from shelter.core.web import BaseRequestHandler


class ShowValueHandler(BaseRequestHandler):

    def compute_etag(self):
        return None

    def get(self):
        self.write("Current value: %s" % self.context.value)
